"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
/* Copyright(C) 2017-2020, HJD (https://github.com/hjdhjd). All rights reserved.
 *
 * blinds-accessory.ts: homebridge-blinds-cmd window covering accessory.
 */
const execa_1 = __importDefault(require("execa"));
class BlindsCmd {
    constructor(log, config, api) {
        this.api = api;
        this.config = config;
        this.log = log;
        // Name these blinds, primarily for logging purposes.
        this.name = config.name || "Blinds";
        // Get our commands to execute.
        this.cmd = { down: config.down, status: config.status, up: config.up };
        // No up or down commands defined, we're done.
        if (!this.cmd.up || !this.cmd.down) {
            this.log("No up or down commands have been configured for this plugin.");
            return;
        }
        // Configure our delay between state changes.
        this.delayTime = config.delay;
        // Configure our status refresh polling.
        this.refreshInterval = config.refresh;
        // Make sure we have a sane value for refresh.
        if (this.refreshInterval < 0) {
            this.refreshInterval = 0;
        }
        // Initialize the blinds. This is a value between 0 - 100, in single steps.
        this.currentPosition = 0;
        this.positionState = api.hap.Characteristic.PositionState.STOPPED;
        this.targetPosition = 0;
        this.configureBlinds();
    }
    async configureBlinds() {
        const Characteristic = this.api.hap.Characteristic;
        const Service = this.api.hap.Service;
        // Create our information service to set some informative parameters for these blinds.
        this.informationService = new Service.AccessoryInformation()
            .setCharacteristic(Characteristic.Manufacturer, this.config.manufacturer || "Default Manufacturer")
            .setCharacteristic(Characteristic.Model, this.config.model || "Default Model")
            .setCharacteristic(Characteristic.Model, this.config.serial || "Default Serial");
        // Now create the window covering service.
        this.blindsService = new Service.WindowCovering(this.name);
        // Initialize our state as stopped.
        this.blindsService.setCharacteristic(Characteristic.PositionState, Characteristic.PositionState.STOPPED);
        // If we have a state command, use it to tell us where we should be on startup.
        if (this.cmd.status) {
            this.currentPosition = await this.execCommand(this.cmd.status);
            // If we had an error getting the initial state, assume the blinds are closed.
            if (this.currentPosition === -1) {
                this.currentPosition = 0;
            }
        }
        // Set the initial position for our blinds.
        this.targetPosition = this.currentPosition;
        this.blindsService.getCharacteristic(Characteristic.CurrentPosition).updateValue(this.currentPosition);
        this.blindsService.getCharacteristic(Characteristic.TargetPosition).updateValue(this.targetPosition);
        // Setup our event listeners.
        this.blindsService
            .getCharacteristic(Characteristic.CurrentPosition)
            .on("get" /* GET */, this.getCurrentPosition.bind(this));
        this.blindsService
            .getCharacteristic(Characteristic.PositionState)
            .on("get" /* GET */, this.getPositionState.bind(this));
        this.blindsService
            .getCharacteristic(Characteristic.TargetPosition)
            .on("get" /* GET */, this.getTargetPosition.bind(this))
            .on("set" /* SET */, this.setTargetPosition.bind(this));
        // Fire off our status polling, if configured.
        if (this.refreshInterval) {
            this.poll();
        }
    }
    // Get the current window covering state.
    async getPositionState(callback) {
        callback(undefined, this.positionState);
    }
    // Get the current window covering state.
    async getCurrentPosition(callback) {
        callback(undefined, this.currentPosition);
    }
    // Get the target window covering state.
    async getTargetPosition(callback) {
        callback(undefined, this.targetPosition);
    }
    // Set the target window covering state and execute the action.
    async setTargetPosition(value, callback) {
        const Characteristic = this.api.hap.Characteristic;
        // Stop polling for status since we're making some user-requested changes.
        if (this.refreshInterval) {
            clearTimeout(this.pollingTimer);
        }
        // We're already where we want to be, do nothing.
        if (value === this.currentPosition) {
            this.targetPosition = value;
            this.positionState = Characteristic.PositionState.STOPPED;
            this.blindsService.getCharacteristic(Characteristic.TargetPosition).updateValue(this.targetPosition);
            this.blindsService.getCharacteristic(Characteristic.CurrentPosition).updateValue(this.currentPosition);
            this.blindsService.getCharacteristic(Characteristic.PositionState).updateValue(this.positionState);
            callback(null);
            this.poll();
            return;
        }
        // Figure out our move dynamics.
        const moveUp = value > this.currentPosition;
        this.targetPosition = value;
        this.positionState = moveUp ? Characteristic.PositionState.INCREASING : Characteristic.PositionState.DECREASING;
        // Tell HomeKit we're on the move.
        this.blindsService.getCharacteristic(Characteristic.PositionState).updateValue(this.positionState);
        this.log((moveUp ? "Moving up." : "Moving down."));
        // Execute the move command.
        let newPosition = await this.execCommand((moveUp ? this.cmd.up : this.cmd.down) + " " + this.targetPosition);
        // Clear out the previous delay timer, if one is configured.
        if (this.delayTime) {
            clearTimeout(this.moveTimer);
        }
        // Something went wrong...cleanup and stop.
        if (newPosition === -1) {
            this.positionState = Characteristic.PositionState.STOPPED;
            this.blindsService.getCharacteristic(Characteristic.PositionState).updateValue(this.positionState);
            callback(Error("Error executing the move command."));
            this.poll();
            return;
        }
        // Special case - if we don't have a script that returns a position as output, we infer the
        // answer based on whether we are opening or closing, and assume we have opened or closed completely.
        if (!newPosition && moveUp) {
            newPosition = 100;
        }
        const self = this;
        // Set a timer to simulate an actual delay in completing the action to give us that interactive feeling.
        this.moveTimer = setTimeout(() => {
            // We've executed all the commands - we're done.
            self.currentPosition = newPosition;
            self.positionState = Characteristic.PositionState.STOPPED;
            self.blindsService.getCharacteristic(Characteristic.TargetPosition).updateValue(self.currentPosition);
            self.blindsService.getCharacteristic(Characteristic.CurrentPosition).updateValue(self.currentPosition);
            self.blindsService.getCharacteristic(Characteristic.PositionState).updateValue(self.positionState);
            self.poll();
        }, this.delayTime * 1000);
        callback(null);
    }
    async poll() {
        const Characteristic = this.api.hap.Characteristic;
        // Clear the last polling interval out.
        clearTimeout(this.pollingTimer);
        // Setup periodic update with our polling interval.
        const self = this;
        this.pollingTimer = setTimeout(async () => {
            if (!self.cmd.status) {
                return;
            }
            // Get our updated state.
            const updatedPosition = await self.execCommand(self.cmd.status);
            // Only update our state if we received a valid status update.
            if (updatedPosition !== -1) {
                self.currentPosition = updatedPosition;
                self.targetPosition = updatedPosition;
                self.blindsService.getCharacteristic(Characteristic.CurrentPosition).updateValue(self.currentPosition);
                self.blindsService.getCharacteristic(Characteristic.TargetPosition).updateValue(self.targetPosition);
            }
            // Fire off the next polling interval.
            if (self.refreshInterval) {
                self.poll();
            }
        }, this.refreshInterval * 1000);
    }
    // Execute a command, with error handling.
    async execCommand(command) {
        try {
            const { stdout } = await execa_1.default.command(command, { shell: true });
            return parseInt(stdout);
        }
        catch (error) {
            this.log("Error executing the command: %s", error.shortMessage);
            return -1;
        }
    }
    // Required method to be called on instantiation. Returns all the services associated with this accessory.
    getServices() {
        return [
            this.informationService,
            this.blindsService
        ];
    }
}
module.exports = (api) => {
    api.registerAccessory("homebridge-blinds-cmd", "Blinds Command", BlindsCmd);
};
//# sourceMappingURL=blinds-accessory.js.map